@@ -1,9 +1,10 @@
 import difflib
-from functools import wraps
+from functools import wraps, partial
 import re
 from flask import request, Response
 from flask import abort as original_flask_abort
 from flask.views import MethodView
+from flask.signals import got_request_exception
 from werkzeug.exceptions import HTTPException
 from flask.ext.restful.utils import unauthorized, error_data, unpack
 from flask.ext.restful.representations.json import output_json
@@ -35,21 +36,70 @@ DEFAULT_REPRESENTATIONS = {'application/json': output_json}
 class Api(object):
     """
     The main entry point for the application.
-    You need to initialize it with a Flask Application :
+    You need to initialize it with a Flask Application:
     >>> app = Flask(__name__)
     >>> api = restful.Api(app)
+
+    Alternatively, you can use :meth:`init_app` to set the Flask application
+    after it has been constructed.
     """
 
-    def __init__(self, app, prefix='', default_mediatype='application/json',
-                 decorators=None):
+    def __init__(self, app=None, prefix='',
+                 default_mediatype='application/json', decorators=None,
+                 catch_all_404s=False):
         self.representations = dict(DEFAULT_REPRESENTATIONS)
         self.urls = {}
         self.prefix = prefix
-        self.app = app
         self.default_mediatype = default_mediatype
         self.decorators = decorators if decorators else []
-        app.handle_exception = self.handle_error
-        app.handle_user_exception = self.handle_error
+        self.catch_all_404s = catch_all_404s
+
+        if app is not None:
+            self.init_app(app)
+        else:
+            self.app = None
+
+    def init_app(self, app):
+        """Initialize this class with the given :class:`flask.Flask`
+        application object.
+
+        :param app: the Flask application object
+        :type app: flask.Flask
+
+        Examples:
+            api = Api()
+            api.init_app(app)
+            api.add_resource(...)
+
+        """
+        self.app = app
+        self.endpoints = set()
+        app.handle_exception = partial(self.error_router, app.handle_exception)
+        app.handle_user_exception = partial(self.error_router, app.handle_user_exception)
+
+    def _has_fr_route(self):
+        """Encapsulating the rules for whether the request was to a Flask endpoint"""
+        return request.url_rule and request.url_rule.endpoint in self.endpoints
+
+    def _is_404_error(self, e):
+        return hasattr(e, "code") and e.code == 404
+
+    def error_router(self, original_handler, e):
+        """This function decides whether the error occured in a flask-restful
+        endpoint or not. If it happened in a flask-restful endpoint, our
+        handler will be dispatched. If it happened in an unrelated view, the
+        app's original error handler will be dispatched.
+
+        :param original_handler: the original Flask error handler for the app
+        :type original_handler: function
+        :param e: the exception raised while handling the request
+        :type e: Exception
+
+        """
+        if self._has_fr_route() or (self.catch_all_404s
+                                    and self._is_404_error(e)):
+            return self.handle_error(e)
+        return original_handler(e)
 
     def handle_error(self, e):
         """Error handler for the API transforms a raised exception into a Flask
@@ -57,7 +107,10 @@ class Api(object):
 
         :param e: the raised Exception object
         :type e: Exception
+
         """
+        got_request_exception.send(self, exception=e)
+
         code = getattr(e, 'code', 500)
         data = getattr(e, 'data', error_data(code))
 
@@ -96,7 +149,7 @@ class Api(object):
         """Adds a resource to the api.
 
         :param resource: the class name of your resource
-        :type resource: Resource
+        :type resource: :class:`Resource`
         :param urls: one or more url routes to match for the resource, standard
                      flask routing rules apply.  Any url variables will be
                      passed to the resource method as args.
@@ -110,14 +163,15 @@ class Api(object):
         Additional keyword arguments not specified above will be passed as-is
         to app.add_url_rule().
 
-        Examples:
-            api.add_resource(HelloWorld, '/', '/hello')
+        Examples::
 
+            api.add_resource(HelloWorld, '/', '/hello')
             api.add_resource(Foo, '/foo', endpoint="foo")
             api.add_resource(FooSpecial, '/special/foo', endpoint="foo")
 
         """
         endpoint = kwargs.pop('endpoint', None) or resource.__name__.lower()
+        self.endpoints.add(endpoint)
 
         if endpoint in self.app.view_functions.keys():
             previous_view_class = self.app.view_functions[endpoint].func_dict['view_class']
@@ -180,7 +234,7 @@ class Api(object):
         The transformer should convert the data appropriately for the mediatype
         and return a Flask response object.
 
-        Ex:
+        Ex::
 
             @api.representation('application/xml')
             def xml(data, code, headers):
@@ -290,5 +344,10 @@ class marshal_with(object):
     def __call__(self, f):
         @wraps(f)
         def wrapper(*args, **kwargs):
-            return marshal(f(*args, **kwargs), self.fields)
+            resp = f(*args, **kwargs)
+            if isinstance(resp, tuple):
+                data, code, headers = unpack(resp)
+                return marshal(data, self.fields), code, headers
+            else:
+                return marshal(resp, self.fields)
         return wrapper
