@@ -26,9 +26,9 @@ def url(value):
     :raises: ValueError
     """
     if not regex.search(value):
-        message = "{0} is not a valid URL".format(value)
+        message = u"{0} is not a valid URL".format(value)
         if regex.search('http://' + value):
-            message += ". Did you mean: http://{0}".format(value)
+            message += u". Did you mean: http://{0}".format(value)
         raise ValueError(message)
     return value
 
