@@ -1,5 +1,6 @@
 import unittest
 from flask import Flask, views
+from flask.signals import got_request_exception, signals_available
 from mock import Mock
 import flask
 import werkzeug
@@ -7,7 +8,7 @@ from flask.ext.restful.utils import http_status_message, challenge, unauthorized
 import flask_restful
 import flask_restful.fields
 from flask_restful import OrderedDict
-from json import dumps
+from json import dumps, loads
 #noinspection PyUnresolvedReferences
 from nose.tools import assert_equals # you need it for tests in form of continuations
 
@@ -21,6 +22,11 @@ def test_unpack():
     yield check_unpack, ("hey", 201, "foo"), unpack(("hey", 201, "foo"))
     yield check_unpack, (["hey", 201], 200, {}), unpack(["hey", 201])
 
+# Add a dummy Resource to verify that the app is properly set.
+class HelloWorld(flask_restful.Resource):
+    def get(self):
+        return {}
+
 class APITestCase(unittest.TestCase):
 
     def test_http_code(self):
@@ -96,6 +102,14 @@ class APITestCase(unittest.TestCase):
             return {'foo': 'bar', 'bat': 'baz'}
         self.assertEquals(try_me(), {'foo': 'bar'})
 
+    def test_marshal_decorator_tuple(self):
+        fields = {'foo': flask_restful.fields.Raw}
+
+        @flask_restful.marshal_with(fields)
+        def try_me():
+            return {'foo': 'bar', 'bat': 'baz'}, 200, {'X-test': 123}
+        self.assertEquals(try_me(), ({'foo': 'bar'}, 200, {'X-test': 123}))
+
     def test_marshal_field(self):
         fields = {'foo': flask_restful.fields.Raw()}
         output = flask_restful.marshal({'foo': 'bar', 'bat': 'baz'}, fields)
@@ -190,6 +204,14 @@ class APITestCase(unittest.TestCase):
         self.assertEquals(api.default_mediatype, 'application/json')
 
 
+    def test_api_delayed_initialization(self):
+        app = Flask(__name__)
+        api = flask_restful.Api()
+        api.init_app(app)
+
+        api.add_resource(HelloWorld, '/', endpoint="hello")
+
+
     def test_api_prefix(self):
         app = Mock()
         api = flask_restful.Api(app, prefix='/foo')
@@ -230,15 +252,61 @@ class APITestCase(unittest.TestCase):
             self.assertTrue('WWW-Authenticate' in resp.headers)
 
 
-    def test_handle_real_error(self):
+    def test_handle_api_error(self):
+        app = Flask(__name__)
+        api = flask_restful.Api(app)
+        class Test(flask_restful.Resource):
+            def get(self):
+                flask.abort(404)
+        api.add_resource(Test(), '/api', endpoint='api')
+        app = app.test_client()
+
+        resp = app.get("/api")
+        self.assertEquals(resp.status_code, 404)
+        self.assertEquals('application/json', resp.headers['Content-Type'])
+        self.assertEquals(loads(resp.data).get('status'), 404)
+
+
+    def test_handle_non_api_error(self):
         app = Flask(__name__)
         flask_restful.Api(app)
         app = app.test_client()
 
         resp = app.get("/foo")
         self.assertEquals(resp.status_code, 404)
-        self.assertEquals(resp.data, dumps(error_data(404)))
+        self.assertEquals('text/html', resp.headers['Content-Type'])
 
+    def test_non_api_error_404_catchall(self):
+        app = Flask(__name__)
+        api = flask_restful.Api(app, catch_all_404s=True)
+        app = app.test_client()
+
+        resp = app.get("/foo")
+        self.assertEquals(api.default_mediatype, resp.headers['Content-Type'])
+
+
+    def test_handle_error_signal(self):
+        if not signals_available:
+            self.skipTest("Can't test signals without signal support")
+        app = Flask(__name__)
+        api = flask_restful.Api(app)
+
+        exception = Mock()
+        exception.code = 400
+        exception.data = {'foo': 'bar'}
+
+        recorded = []
+        def record(sender, exception):
+            recorded.append(exception)
+
+        got_request_exception.connect(record, api)
+        try:
+            with app.test_request_context("/foo"):
+                api.handle_error(exception)
+                self.assertEquals(len(recorded), 1)
+                self.assertTrue(exception is recorded[0])
+        finally:
+            got_request_exception.disconnect(record, app)
 
     def test_handle_error(self):
         app = Flask(__name__)
@@ -501,5 +569,17 @@ class APITestCase(unittest.TestCase):
     def test_abort_type(self):
         self.assertRaises(werkzeug.exceptions.HTTPException, lambda: flask_restful.abort(404))
 
+
+    def test_endpoints(self):
+        app = Flask(__name__)
+        api = flask_restful.Api(app)
+        api.add_resource(HelloWorld, '/ids/<int:id>', endpoint="hello")
+        with app.test_request_context('/foo'):
+            self.assertFalse(api._has_fr_route())
+
+        with app.test_request_context('/ids/3'):
+            self.assertTrue(api._has_fr_route())
+
+
 if __name__ == '__main__':
     unittest.main()
